/*- - - - - - - - - - - - - - - - - - - - - - - - - - - -
	INTLDATE.CPP

	TIntlDate Class

	Author:     Daniel BERMAN (CompuServe: 100567,3464)
	Date:       13 jully 1995 (1.01)

*/

#include "intldate.h"


static const unsigned char daysInMonth[12] =
	{31,28,31,30,31,30,31,31,30,31,30,31};

static const char* MonthNames[1][12] =
	{
		{"janvier","fvrier","mars","avril","mai","juin",
		 "juillet","aot","septembre","octobre","novembre","dcembre"}
	};

static const char* UCMonthNames[1][12] =
	{
		{"JANVIER","FEVRIER","MARS","AVRIL","MAI","JUIN",
		 "JUILLET","AOUT","SEPTEMBRE","OCTOBRE","NOVEMBRE","DECEMBRE"}
	};

static const char* WeekDayNames[1][7] =
	{
		{"lundi","mardi","mercredi","jeudi","vendredi","samedi","dimanche"}
	};


static int FindMatch(const char* str, const char** candidates, int icand);
static void SkipDelim(istream& strm);

TIntlDate::HowToPrint TIntlDate::PrintOption = TIntlDate::Normal;
TIntlDate::HowToSpeak TIntlDate::Language = English;


// Construct a TIntlDate for today's date.
TIntlDate::TIntlDate()
{
	Date = new TDate();
}

TIntlDate::TIntlDate(DayTy day, MonthTy month, YearTy year)
{
	Date = new TDate(day, month, year);
}

TIntlDate::TIntlDate(DayTy day, const char* month, YearTy year)
{
	Date = new TDate(day, month, year);
}

TIntlDate::TIntlDate(const TDate& date)
{
	Date = new TDate(date);
}

TIntlDate::TIntlDate(istream& s)
{
	switch (Language) {
		case English:
		default:
			if (PrintOption !=  TIntlDate::European &&
				 PrintOption !=  TIntlDate::EuropeanNumbers) {
				Date = new TDate(s);
				break;
			}
		case French:
			EuropeanParseFrom(s);
	}
}

TIntlDate::~TIntlDate()
{
	delete Date;
}

string TIntlDate::AsString() const
{
	char buf[80];
	ostrstream strtemp(buf, sizeof(buf));

	strtemp << *this << ends;
	string temp(buf);

	return temp;
}

// Returns a string name for the weekday number.
// Monday == 1, ... , Sunday == 7
// Return 0 for weekday number out of range
const char* TIntlDate::DayName(DayTy weekDayNumber)
{
	if (!AssertWeekDayNumber(weekDayNumber))
		return 0;

	switch (Language) {
		case French:
			return WeekDayNames[French][weekDayNumber-1];
		case English:
		default:
			return TDate::DayName(weekDayNumber);
	}
}

// Return the number of days of a given month
// Return 0 if "month" is outside of the range 1 through 12, inclusive.
int TIntlDate::DaysInMonth(MonthTy month) const
{
	if ( !AssertIndexOfMonth(month) )
		  return 0;

	unsigned days = daysInMonth[month-1];
	if(Leap() && month == 2)
		days++;

	return days;
}

// Returns the number, 1-12, of the month named nameOfMonth.
// Return 0 for no match.
MonthTy TIntlDate::IndexOfMonth(const char* nameOfMonth)
{
	switch (Language) {
		case French:
    		return (MonthTy)(FindMatch(nameOfMonth, UCMonthNames[French], 12)+1);
		case English:
		default:
			return TDate::IndexOfMonth(nameOfMonth);
	}
}

// Returns a string name for the month number.
// Return 0 if invalid month number.
const char* TIntlDate::MonthName(MonthTy monthNumber)
{
	if (!AssertIndexOfMonth(monthNumber))
		return 0;

	switch (Language) {
		case French:
			return MonthNames[French][monthNumber-1];
		case English:
		default:
			return TDate::MonthName(monthNumber);
	}
}

// Parse a date from the specified input stream.
//  The date must be in the following form:
//   dd/mm/yy
//    e.g.: 13/7/95, 13-07-1995.
// Any non-alphanumeric character may be used as a delimiter.
void TIntlDate::EuropeanParseFrom(istream& s)
{
	unsigned int d, m, y;

	if (s.good()) {
		SkipDelim(s);
		s >> d ;
		SkipDelim(s);
		if (!s.eof() && s.good()) {
			s >> m;
			if (!s.eof() && s.good()) {
				SkipDelim(s);
				s >> y;
			}
		}
	}
	Date = new TDate(d, m, y);
	if(!Date->IsValid())
		s.clear(ios::badbit);
}

TIntlDate::HowToSpeak TIntlDate::SetLanguageOption(HowToSpeak l)
{
	HowToSpeak old = Language;
	Language = l;
	return old;
}

TIntlDate::HowToPrint TIntlDate::SetPrintOption(TIntlDate::HowToPrint h)
{
	TIntlDate::HowToPrint old = PrintOption;
	PrintOption = h;
	return old;
}

// Returns the first 2 caracters name for the weekday number.
// Mo == 1, ... , Su == 7
// Return string of length 0 for weekday number out of range
const string TIntlDate::ShortDayName(DayTy weekDayNumber)
{
	const char* name = DayName(weekDayNumber);

	if (!name)
		return string();

	return string(name, 0, 2);
}

// Returns 1 (Monday) through 7 (Sunday) for DayTy of this TIntlDate.
// Return 0 if invalid day.
DayTy TIntlDate::WeekDay(DayTy day) const
{
	if (!AssertWeekDayNumber(day))
		return DayTy(0);

	return TDate(day, Month(), Year()).WeekDay();
}

TIntlDate& const TIntlDate::operator = (const TIntlDate& dt)
{
	delete Date;
	Date = new TDate(*dt.Date);
	return *this;
}

TIntlDate& const TIntlDate::operator = (const TDate& dt)
{
	delete Date;
	Date = new TDate(dt);
	return *this;
}

ostream& operator << (ostream& s, const TIntlDate& d)
{
	char buf[80];

	// we use an ostrstream to format into buf so that
	// we don't affect the ostream's width setting.
	ostrstream out(buf, sizeof(buf));

	switch (TIntlDate::Language) {
		case TIntlDate::French:
			switch (TIntlDate::PrintOption) {
				case TIntlDate::Normal:
					out << d.NameOfDay()   << " "
						 << d.DayOfMonth()  << " "
						 << d.NameOfMonth() << " "
						 << d.Year()        << ends;
					break;
				case TIntlDate::Terse:
				case TIntlDate::European:
					out << d.DayOfMonth()  << " "
						 << d.NameOfMonth() << " "
						 << d.Year()        << ends;
					break;
				case TIntlDate::Numbers:
				case TIntlDate::EuropeanNumbers:
					out << d.DayOfMonth()   << "/"
						 << d.Month()        <<"/"
						 << (d.Year() % 100) << ends;
					break;
			};
			break;
		case TIntlDate::English:
		default:
			TDate::HowToPrint old = d.Date->SetPrintOption((TDate::HowToPrint)TIntlDate::PrintOption);
			out << *(d.Date) << ends;
			d.Date->SetPrintOption(old);
	}

	// now we write out the formatted buffer, and the ostream's
	// width setting will control the actual width of the field.
	s << buf;
	return s;
}

// Return index of case-insensitive match; -1 if no match.
static int FindMatch(const char* str, const char** candidates, int icand)
{
	char* tmp;
	unsigned len = strlen(str);

	tmp = new char[len];
   strcpy(tmp, str);
	AnsiUpperBuff(tmp, len);

	while(icand--)
		if(strncmpi(tmp, candidates[icand], len) == 0)
			break;

	delete [] tmp;

	return icand;
}

// Skip any characters except alphanumeric characters
static void SkipDelim(istream& strm)
{
	char c;
	if(!strm.good())
		return;

	do {
		strm >> c;
	} while (strm.good() && !isalnum(c)) ;

	if (strm.good())
		strm.putback(c);
}



